@@ -69,17 +69,20 @@ enum {
 };
 
 enum {
-        BEGIN_TIMED_LOGIN,
+        BEGIN_AUTO_LOGIN,
         BEGIN_VERIFICATION,
         BEGIN_VERIFICATION_FOR_USER,
         QUERY_ANSWER,
         SESSION_SELECTED,
         HOSTNAME_SELECTED,
         LANGUAGE_SELECTED,
+        LAYOUT_SELECTED,
         USER_SELECTED,
         CANCELLED,
         CONNECTED,
         DISCONNECTED,
+        START_SESSION_WHEN_READY,
+        START_SESSION_LATER,
         LAST_SIGNAL
 };
 
@@ -117,6 +120,40 @@ send_dbus_message (DBusConnection *connection,
         return sent;
 }
 
+static void
+send_dbus_string_and_int_signal (GdmGreeterServer *greeter_server,
+                                 const char       *name,
+                                 const char       *text,
+                                 int               number)
+{
+        DBusMessage    *message;
+        DBusMessageIter iter;
+        const char     *str;
+
+        if (text != NULL) {
+                str = text;
+        } else {
+                str = "";
+        }
+
+        g_return_if_fail (greeter_server != NULL);
+
+        message = dbus_message_new_signal (GDM_GREETER_SERVER_DBUS_PATH,
+                                           GDM_GREETER_SERVER_DBUS_INTERFACE,
+                                           name);
+
+        dbus_message_iter_init_append (message, &iter);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_STRING, &str);
+        dbus_message_iter_append_basic (&iter, DBUS_TYPE_INT32, &number);
+
+        g_debug ("GreeterServer: Sending %s (%s %d)", name, str, number);
+        if (! send_dbus_message (greeter_server->priv->greeter_connection, message)) {
+                g_debug ("GreeterServer: Could not send %s signal", name);
+        }
+
+        dbus_message_unref (message);
+}
+
 static void
 send_dbus_string_signal (GdmGreeterServer *greeter_server,
                          const char       *name,
@@ -229,6 +266,13 @@ gdm_greeter_server_default_language_name_changed (GdmGreeterServer *greeter_serv
         send_dbus_string_signal (greeter_server, "DefaultLanguageNameChanged", language_name);
 }
 
+void
+gdm_greeter_server_default_layout_name_changed (GdmGreeterServer *greeter_server,
+                                                const char       *layout_name)
+{
+        send_dbus_string_signal (greeter_server, "DefaultLayoutNameChanged", layout_name);
+}
+
 void
 gdm_greeter_server_default_session_name_changed (GdmGreeterServer *greeter_server,
                                                  const char       *session_name)
@@ -236,6 +280,20 @@ gdm_greeter_server_default_session_name_changed (GdmGreeterServer *greeter_serve
         send_dbus_string_signal (greeter_server, "DefaultSessionNameChanged", session_name);
 }
 
+void
+gdm_greeter_server_request_timed_login (GdmGreeterServer *greeter_server,
+                                        const char       *username,
+                                        int               delay)
+{
+        send_dbus_string_and_int_signal (greeter_server, "TimedLoginRequested", username, delay);
+}
+
+void
+gdm_greeter_server_user_authorized (GdmGreeterServer *greeter_server)
+{
+        send_dbus_void_signal (greeter_server, "UserAuthorized");
+}
+
 /* Note: Use abstract sockets like dbus does by default on Linux. Abstract
  * sockets are only available on Linux.
  */
@@ -283,19 +341,29 @@ handle_begin_verification (GdmGreeterServer *greeter_server,
 }
 
 static DBusHandlerResult
-handle_begin_timed_login (GdmGreeterServer *greeter_server,
-                        DBusConnection   *connection,
-                        DBusMessage      *message)
+handle_begin_auto_login (GdmGreeterServer *greeter_server,
+                         DBusConnection   *connection,
+                         DBusMessage      *message)
 {
         DBusMessage *reply;
+        DBusError    error;
+        const char  *text;
 
-        g_debug ("GreeterServer: BeginTimedLogin");
+
+        dbus_error_init (&error);
+        if (! dbus_message_get_args (message, &error,
+                                     DBUS_TYPE_STRING, &text,
+                                     DBUS_TYPE_INVALID)) {
+                g_warning ("ERROR: %s", error.message);
+        }
+
+        g_debug ("GreeterServer: BeginAutoLogin for '%s'", text);
 
         reply = dbus_message_new_method_return (message);
         dbus_connection_send (connection, reply, NULL);
         dbus_message_unref (reply);
 
-        g_signal_emit (greeter_server, signals [BEGIN_TIMED_LOGIN], 0);
+        g_signal_emit (greeter_server, signals [BEGIN_AUTO_LOGIN], 0, text);
 
         return DBUS_HANDLER_RESULT_HANDLED;
 }
@@ -435,6 +503,33 @@ handle_select_language (GdmGreeterServer *greeter_server,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+handle_select_layout (GdmGreeterServer *greeter_server,
+                      DBusConnection  *connection,
+                      DBusMessage     *message)
+{
+        DBusMessage *reply;
+        DBusError    error;
+        const char  *text;
+
+        dbus_error_init (&error);
+        if (! dbus_message_get_args (message, &error,
+                                     DBUS_TYPE_STRING, &text,
+                                     DBUS_TYPE_INVALID)) {
+                g_warning ("ERROR: %s", error.message);
+        }
+
+        g_debug ("GreeterServer: SelectLayout: %s", text);
+
+        reply = dbus_message_new_method_return (message);
+        dbus_connection_send (connection, reply, NULL);
+        dbus_message_unref (reply);
+
+        g_signal_emit (greeter_server, signals [LAYOUT_SELECTED], 0, text);
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static DBusHandlerResult
 handle_select_user (GdmGreeterServer *greeter_server,
                     DBusConnection   *connection,
@@ -511,6 +606,38 @@ handle_get_display_id (GdmGreeterServer *greeter_server,
         return DBUS_HANDLER_RESULT_HANDLED;
 }
 
+static DBusHandlerResult
+handle_start_session_when_ready (GdmGreeterServer *greeter_server,
+                                 DBusConnection   *connection,
+                                 DBusMessage      *message)
+{
+        DBusMessage *reply;
+        DBusError    error;
+        gboolean     should_start_session;
+
+        dbus_error_init (&error);
+        if (! dbus_message_get_args (message, &error,
+                                     DBUS_TYPE_BOOLEAN, &should_start_session,
+                                     DBUS_TYPE_INVALID)) {
+                g_warning ("ERROR: %s", error.message);
+        }
+
+        g_debug ("GreeterServer: %sStartSessionWhenReady",
+                 should_start_session? "" : "Don't ");
+
+        reply = dbus_message_new_method_return (message);
+        dbus_connection_send (connection, reply, NULL);
+        dbus_message_unref (reply);
+
+        if (should_start_session) {
+                g_signal_emit (greeter_server, signals [START_SESSION_WHEN_READY], 0);
+        } else {
+                g_signal_emit (greeter_server, signals [START_SESSION_LATER] ,0);
+        }
+
+        return DBUS_HANDLER_RESULT_HANDLED;
+}
+
 static DBusHandlerResult
 greeter_handle_child_message (DBusConnection *connection,
                               DBusMessage    *message,
@@ -522,8 +649,8 @@ greeter_handle_child_message (DBusConnection *connection,
                 return handle_begin_verification (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "BeginVerificationForUser")) {
                 return handle_begin_verification_for_user (greeter_server, connection, message);
-        } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "BeginTimedLogin")) {
-                return handle_begin_timed_login (greeter_server, connection, message);
+        } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "BeginAutoLogin")) {
+                return handle_begin_auto_login (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "AnswerQuery")) {
                 return handle_answer_query (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "SelectSession")) {
@@ -532,6 +659,8 @@ greeter_handle_child_message (DBusConnection *connection,
                 return handle_select_hostname (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "SelectLanguage")) {
                 return handle_select_language (greeter_server, connection, message);
+        } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "SelectLayout")) {
+                return handle_select_layout (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "SelectUser")) {
                 return handle_select_user (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "Cancel")) {
@@ -540,6 +669,8 @@ greeter_handle_child_message (DBusConnection *connection,
                 return handle_disconnect (greeter_server, connection, message);
         } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "GetDisplayId")) {
                 return handle_get_display_id (greeter_server, connection, message);
+        } else if (dbus_message_is_method_call (message, GDM_GREETER_SERVER_DBUS_INTERFACE, "StartSessionWhenReady")) {
+                return handle_start_session_when_ready (greeter_server, connection, message);
         }
 
         return DBUS_HANDLER_RESULT_NOT_YET_HANDLED;
@@ -596,6 +727,9 @@ do_introspect (DBusConnection *connection,
                                "    <method name=\"GetDisplayId\">\n"
                                "      <arg name=\"id\" direction=\"out\" type=\"o\"/>\n"
                                "    </method>\n"
+                               "    <method name=\"StartSessionWhenReady\">\n"
+                               "      <arg name=\"should_start_session\" type=\"b\"/>\n"
+                               "    </method>\n"
                                "    <signal name=\"Info\">\n"
                                "      <arg name=\"text\" type=\"s\"/>\n"
                                "    </signal>\n"
@@ -614,13 +748,22 @@ do_introspect (DBusConnection *connection,
                                "    <signal name=\"DefaultLanguageNameChanged\">\n"
                                "      <arg name=\"language_name\" type=\"s\"/>\n"
                                "    </signal>\n"
+                               "    <signal name=\"DefaultLayoutNameChanged\">\n"
+                               "      <arg name=\"layout_name\" type=\"s\"/>\n"
+                               "    </signal>\n"
                                "    <signal name=\"DefaultSessionNameChanged\">\n"
                                "      <arg name=\"session_name\" type=\"s\"/>\n"
                                "    </signal>\n"
+                               "    <signal name=\"TimedLoginRequested\">\n"
+                               "      <arg name=\"username\" type=\"s\"/>\n"
+                               "      <arg name=\"delay\" type=\"i\"/>\n"
+                               "    </signal>\n"
                                "    <signal name=\"Ready\">\n"
                                "    </signal>\n"
                                "    <signal name=\"Reset\">\n"
                                "    </signal>\n"
+                               "    <signal name=\"UserAuthorized\">\n"
+                               "    </signal>\n"
                                "  </interface>\n");
 
         reply = dbus_message_new_method_return (message);
@@ -969,14 +1112,14 @@ gdm_greeter_server_class_init (GdmGreeterServerClass *klass)
                                          g_param_spec_string ("user-name",
                                                               "user name",
                                                               "user name",
-                                                              "gdm",
+                                                              GDM_USERNAME,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
         g_object_class_install_property (object_class,
                                          PROP_GROUP_NAME,
                                          g_param_spec_string ("group-name",
                                                               "group name",
                                                               "group name",
-                                                              "gdm",
+                                                              GDM_GROUPNAME,
                                                               G_PARAM_READWRITE | G_PARAM_CONSTRUCT));
         signals [BEGIN_VERIFICATION] =
                 g_signal_new ("begin-verification",
@@ -988,16 +1131,17 @@ gdm_greeter_server_class_init (GdmGreeterServerClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
-        signals [BEGIN_TIMED_LOGIN] =
-                g_signal_new ("begin-timed-login",
+        signals [BEGIN_AUTO_LOGIN] =
+                g_signal_new ("begin-auto-login",
                               G_OBJECT_CLASS_TYPE (object_class),
                               G_SIGNAL_RUN_FIRST,
-                              G_STRUCT_OFFSET (GdmGreeterServerClass, begin_timed_login),
+                              G_STRUCT_OFFSET (GdmGreeterServerClass, begin_auto_login),
                               NULL,
                               NULL,
-                              g_cclosure_marshal_VOID__VOID,
+                              g_cclosure_marshal_VOID__STRING,
                               G_TYPE_NONE,
-                              0);
+                              1,
+                              G_TYPE_STRING);
         signals [BEGIN_VERIFICATION_FOR_USER] =
                 g_signal_new ("begin-verification-for-user",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -1053,6 +1197,17 @@ gdm_greeter_server_class_init (GdmGreeterServerClass *klass)
                               G_TYPE_NONE,
                               1,
                               G_TYPE_STRING);
+        signals [LAYOUT_SELECTED] =
+                g_signal_new ("layout-selected",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterServerClass, layout_selected),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__STRING,
+                              G_TYPE_NONE,
+                              1,
+                              G_TYPE_STRING);
         signals [USER_SELECTED] =
                 g_signal_new ("user-selected",
                               G_OBJECT_CLASS_TYPE (object_class),
@@ -1094,6 +1249,28 @@ gdm_greeter_server_class_init (GdmGreeterServerClass *klass)
                               g_cclosure_marshal_VOID__VOID,
                               G_TYPE_NONE,
                               0);
+
+        signals [START_SESSION_WHEN_READY] =
+                g_signal_new ("start-session-when-ready",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterServerClass, start_session_when_ready),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
+
+        signals [START_SESSION_LATER] =
+                g_signal_new ("start-session-later",
+                              G_OBJECT_CLASS_TYPE (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GdmGreeterServerClass, start_session_later),
+                              NULL,
+                              NULL,
+                              g_cclosure_marshal_VOID__VOID,
+                              G_TYPE_NONE,
+                              0);
 }
 
 static void
